==> gm-1.6.3_Linux.tar.gz: gm-1.6.3_Linux/drivers/nt5.0/gm_miniport/gmm.h <==
#ifndef _gmm_h_
#define _gmm_h_

#include "gm_internal.h"
#include "gm_ether.h"

#define GM_MINIPORT_MAJOR_VERSION 5
#define GM_MINIPORT_MINOR_VERSION 0

#define GMM_NUM_SEND_BUFFERS 10
#define GMM_NUM_RECV_BUFFERS GM_NUM_ETHERNET_RECV_TOKENS

#define GMM_MAX_PACKETS_PER_SEND GMM_NUM_SEND_BUFFERS	/*per send */

#define GMM_ETHERNET_HEADER_SIZE 14
#define GMM_HEADER_SIZE 2
#define GMM_SLOP (GMM_HEADER_SIZE + GMM_ETHERNET_HEADER_SIZE + 40)

/*#define GMM_MTU 1514*/
#define GMM_MTU (GM_IP_MTU+GMM_SLOP)

#define GMM_VENDOR_DESCRIPTION "GM Miniport"
#define GMM_DRIVER_VERSION 1
#define GMM_MAX_MULTICAST_ADDRESSES 32 /*0*/	/*todo */
#define GMM_PORT_NAME "GM Miniport"

#define GMM_NUM_MAP_REGISTERS (GMM_NUM_SEND_BUFFERS * GM_MAX_ETHERNET_GATHER_CNT)
#define GMM_MAX_BYTES_PER_DMA 4096

typedef struct gmm_send_item
{
  char *va;
  NDIS_PHYSICAL_ADDRESS pa;
  PNDIS_PACKET packet;
}
gmm_send_item;

typedef struct gmm_recv_item
{
  char *va;
  NDIS_PHYSICAL_ADDRESS pa;
  PNDIS_PACKET packet;
  PNDIS_BUFFER buffer;
}
gmm_recv_item;

typedef struct gmm_Context
{
  NDIS_HANDLE miniportAdapterHandle;
  gm_instance_state_t *is;
  struct gm_port *port;
  ULONG cacheFillSize;

  NDIS_SPIN_LOCK send_lock;
  gmm_send_item send_ring[GMM_NUM_SEND_BUFFERS];
  int send_ring_head;
  int send_ring_count;
  PNDIS_PACKET firstTxQueue;
  PNDIS_PACKET lastTxQueue;
  UINT numPacketsQueued;

  NDIS_HANDLE recv_packet_pool;
  NDIS_HANDLE recv_buffer_pool;

  NDIS_SPIN_LOCK recv_lock;
  gmm_recv_item *recv_ring[GMM_NUM_RECV_BUFFERS];
  gmm_recv_item recv_item[GMM_NUM_RECV_BUFFERS];
  int recv_ring_head;
  int recv_ring_count;

  ULONG xmit_ok;
  ULONG rcv_ok;
  ULONG xmit_error;
  ULONG rcv_error;
}
gmm_Context;

typedef struct _GMM_RESERVED
{
  PNDIS_PACKET next;
}
GMM_RESERVED, *PGMM_RESERVED;

#define PGMM_RESERVED_FROM_PACKET(_packet) \
    ((PGMM_RESERVED)((_packet)->MiniportReserved))

#define ENQUEUE_PACKET(_head, _tail, _packet)          \
{                                                      \
  if (!_head)                                          \
  {                                                    \
    _head = _packet;                                   \
  }                                                    \
  else                                                 \
  {                                                    \
    PGMM_RESERVED_FROM_PACKET(_tail)->next = _packet;  \
  }                                                    \
  PGMM_RESERVED_FROM_PACKET(_packet)->next = NULL;     \
  _tail = _packet;                                     \
}

#define DEQUEUE_PACKET(head, tail)                     \
{                                                      \
  PGMM_RESERVED reserved =                             \
    PGMM_RESERVED_FROM_PACKET(head);                   \
  if (!reserved->next)                                 \
  {                                                    \
    tail = NULL;                                       \
  }                                                    \
  head = reserved->next;                               \
}

NDIS_STATUS gmm_Initialize (PNDIS_STATUS openErrorStatus,
			    PUINT SelectedMediumIndex,
			    PNDIS_MEDIUM MediumArray,
			    UINT MediumArraySize,
			    NDIS_HANDLE MiniportAdapterHandle,
			    NDIS_HANDLE WrapperConfigurationContext);

VOID gmm_Unload (IN PDRIVER_OBJECT DriverObject);

NDIS_STATUS gmm_SetInformation (NDIS_HANDLE miniportAdapterContext,
				NDIS_OID oid,
				PVOID informationBuffer,
				ULONG informationBufferLength,
				PULONG bytesRead, PULONG bytesNeeded);

NDIS_STATUS gmm_QueryInformation (NDIS_HANDLE miniportAdapterContext,
				  NDIS_OID oid,
				  PVOID informationBuffer,
				  ULONG informationBufferLength,
				  PULONG bytesWritten, PULONG bytesNeeded);

void gmm_SendPackets (NDIS_HANDLE miniportAdapterContext,
		      PPNDIS_PACKET packets, UINT numPackets);


VOID gmm_ReturnPacket (NDIS_HANDLE miniportAdapterContext,
		       PNDIS_PACKET packet);

BOOLEAN gmm_CheckForHang (IN NDIS_HANDLE miniportAdapterContext);
VOID gmm_Halt (NDIS_HANDLE miniportAdapterContext);
NDIS_STATUS gmm_Reset (PBOOLEAN addressingReset,
		       NDIS_HANDLE miniportAdapterContext);


void gmm_free (void *p, UINT size);
void *gmm_calloc (UINT size);
void gmm_receiveInterrupt (gmm_Context * context, unsigned length,
			   gm_u16_t csum);
void gmm_sendInterrupt (gmm_Context * context);

NDIS_STATUS gmm_allocate_send_ring (gmm_Context * context);
void gmm_free_send_ring (gmm_Context * context);

NDIS_STATUS gmm_allocate_recv_ring (gmm_Context * context);
void gmm_provide_recv_ring (gmm_Context * context);
void gmm_free_recv_ring (gmm_Context * context);

#ifdef GMM_ENABLE_INSIST
#define insist(e)							\
do									\
{									\
  if (!(e))								\
    {									\
      DbgPrint ("gm_miniport: assertion failed in file %s, line %d\n",	\
		__FILE__,__LINE__);					\
      goto exception;							\
    }									\
}									\
while (0)
#else
#define insist(e) if(0) goto exception
#endif

#ifdef GMM_ENABLE_PRINT
#define GMM_PRINT(args)							\
do									\
{									\
  DbgPrint ("gm_miniport: ");				\
  DbgPrint args;							\
}									\
while (0)
#else
#define GMM_PRINT(args)
#endif

#define GMM_ALWAYS_PRINT(args)						\
do									\
{									\
  DbgPrint ("gm_miniport: ");						\
  DbgPrint args;							\
}									\
while (0)


#endif /*_gmm_h_*/

==> gm-2.0.8_Linux.tar.gz: gm-2.0.8_Linux/drivers/nt5.0/gm_miniport/gmm.h <==
#ifndef _gmm_h_
#define _gmm_h_

#include "gm_internal.h"
#include "gm_ether.h"

#define GM_MINIPORT_MAJOR_VERSION 5
#define GM_MINIPORT_MINOR_VERSION 0

#define GMM_NUM_SEND_BUFFERS 10
#define GMM_NUM_RECV_BUFFERS GM_NUM_ETHERNET_RECV_TOKENS

#define GMM_MAX_PACKETS_PER_SEND GMM_NUM_SEND_BUFFERS	/*per send */

#define GMM_ETHERNET_HEADER_SIZE 14

#define GMM_VENDOR_DESCRIPTION "GM Miniport"
#define GMM_DRIVER_VERSION 1
#define GMM_MAX_MULTICAST_ADDRESSES 32 /*0*/	/*todo */
#define GMM_PORT_NAME "GM Miniport"

#define GMM_NUM_MAP_REGISTERS (GMM_NUM_SEND_BUFFERS * GM_MAX_ETHERNET_GATHER_CNT)
#define GMM_MAX_BYTES_PER_DMA 4096

typedef struct gmm_send_item
{
  char *va;
  NDIS_PHYSICAL_ADDRESS pa;
  PNDIS_PACKET packet;
}
gmm_send_item;

typedef struct gmm_recv_item
{
  char *va;
  NDIS_PHYSICAL_ADDRESS pa;
  PNDIS_PACKET packet;
  PNDIS_BUFFER buffer;
  int offset;
}
gmm_recv_item;

typedef struct gmm_Context
{
  NDIS_HANDLE miniportAdapterHandle;
  gm_instance_state_t *is;
  struct gm_port *port;
  ULONG cacheFillSize;

  NDIS_SPIN_LOCK send_lock;
  gmm_send_item send_ring[GMM_NUM_SEND_BUFFERS];
  int send_ring_head;
  int send_ring_count;
  PNDIS_PACKET firstTxQueue;
  PNDIS_PACKET lastTxQueue;
  UINT numPacketsQueued;

  NDIS_HANDLE recv_packet_pool;
  NDIS_HANDLE recv_buffer_pool;

  NDIS_SPIN_LOCK recv_lock;
  gmm_recv_item recv_ring[GMM_NUM_RECV_BUFFERS];
  int recv_ring_count;

  ULONG xmit_ok;
  ULONG rcv_ok;
  ULONG xmit_error;
  ULONG rcv_error;
}
gmm_Context;

typedef struct _GMM_RESERVED
{
  PNDIS_PACKET next;
}
GMM_RESERVED, *PGMM_RESERVED;

#define PGMM_RESERVED_FROM_PACKET(_packet) \
    ((PGMM_RESERVED)((_packet)->MiniportReserved))

#define ENQUEUE_PACKET(_head, _tail, _packet)          \
{                                                      \
  if (!_head)                                          \
  {                                                    \
    _head = _packet;                                   \
  }                                                    \
  else                                                 \
  {                                                    \
    PGMM_RESERVED_FROM_PACKET(_tail)->next = _packet;  \
  }                                                    \
  PGMM_RESERVED_FROM_PACKET(_packet)->next = NULL;     \
  _tail = _packet;                                     \
}

#define DEQUEUE_PACKET(head, tail)                     \
{                                                      \
  PGMM_RESERVED reserved =                             \
    PGMM_RESERVED_FROM_PACKET(head);                   \
  if (!reserved->next)                                 \
  {                                                    \
    tail = NULL;                                       \
  }                                                    \
  head = reserved->next;                               \
}

NDIS_STATUS gmm_Initialize (PNDIS_STATUS openErrorStatus,
			    PUINT SelectedMediumIndex,
			    PNDIS_MEDIUM MediumArray,
			    UINT MediumArraySize,
			    NDIS_HANDLE MiniportAdapterHandle,
			    NDIS_HANDLE WrapperConfigurationContext);

VOID gmm_Unload (IN PDRIVER_OBJECT DriverObject);

NDIS_STATUS gmm_SetInformation (NDIS_HANDLE miniportAdapterContext,
				NDIS_OID oid,
				PVOID informationBuffer,
				ULONG informationBufferLength,
				PULONG bytesRead, PULONG bytesNeeded);

NDIS_STATUS gmm_QueryInformation (NDIS_HANDLE miniportAdapterContext,
				  NDIS_OID oid,
				  PVOID informationBuffer,
				  ULONG informationBufferLength,
				  PULONG bytesWritten, PULONG bytesNeeded);

void gmm_SendPackets (NDIS_HANDLE miniportAdapterContext,
		      PPNDIS_PACKET packets, UINT numPackets);


VOID gmm_ReturnPacket (NDIS_HANDLE miniportAdapterContext,
		       PNDIS_PACKET packet);

BOOLEAN gmm_CheckForHang (IN NDIS_HANDLE miniportAdapterContext);
VOID gmm_Halt (NDIS_HANDLE miniportAdapterContext);
NDIS_STATUS gmm_Reset (PBOOLEAN addressingReset,
		       NDIS_HANDLE miniportAdapterContext);


void gmm_free (void *p, UINT size);
void *gmm_calloc (UINT size);
void gmm_receiveInterrupt (void *p, gm_ethernet_recv_t *e);
void gmm_sendInterrupt (gmm_Context * context);

NDIS_STATUS gmm_allocate_send_ring (gmm_Context * context);
void gmm_free_send_ring (gmm_Context * context);

NDIS_STATUS gmm_allocate_recv_ring (gmm_Context * context);
void gmm_provide_recv_ring (gmm_Context * context);
void gmm_free_recv_ring (gmm_Context * context);

#ifdef GMM_ENABLE_INSIST
#define insist(e)							\
do									\
{									\
  if (!(e))								\
    {									\
      DbgPrint ("gm_miniport: assertion failed in file %s, line %d\n",	\
		__FILE__,__LINE__);					\
      goto exception;							\
    }									\
}									\
while (0)
#else
#define insist(e) if(0) goto exception
#endif

#ifdef GMM_ENABLE_PRINT
#define GMM_PRINT(args)							\
do									\
{									\
  DbgPrint ("gm_miniport: ");				\
  DbgPrint args;							\
}									\
while (0)
#else
#define GMM_PRINT(args)
#endif

#define GMM_ALWAYS_PRINT(args)						\
do									\
{									\
  DbgPrint ("gm_miniport: ");						\
  DbgPrint args;							\
}									\
while (0)


#endif /*_gmm_h_*/
